/* global OpenDocument */

/***********************************************************
 * 
 * @constructor
 * @param {String} styleName
 * @param {String} styleFamily
 * @param {String} parentStyleName
 * @returns {OpenDocument.Style}
 */
OpenDocument.Style = function (styleName, styleFamily, parentStyleName) {
    this.styleName = styleName;
    this.styleFamily = styleFamily;
    this.parentStyleName = parentStyleName;
    this.paragraphArray = new Array();
    this.textArray = new Array();
    this.cellArray = new Array();
    this.rowArray = new Array();
    this.columnArray = new Array();
};

OpenDocument.Style.prototype.add = function (attrDef, value) {
    var attr = {
        name: attrDef.name,
        value: value
    };
    switch(attrDef.type) {
        case "paragraph":
            this.paragraphArray.push(attr);
            break;
        case "text":
            this.textArray.push(attr);
            break;
        case "cell":
            this.cellArray.push(attr);
            break;
        case "row":
            this.rowArray.push(attr);
            break;
        case "column":
            this.columnArray.push(attr);
            break;
    }
};

/**
 * 
 * @param {OpenDocument.XmlWriter} xmlWriter
 * @returns {undefined}
 */
OpenDocument.Style.prototype.write = function (xmlWriter) {
    xmlWriter.openStyle(this.styleName, this.styleFamily, this.parentStyleName);
    _properties(this.paragraphArray, "style:paragraph-properties");
    _properties(this.textArray, "style:text-properties");
    _properties(this.cellArray, "style:table-cell-properties");
    _properties(this.rowArray, "style:table-row-properties");
    _properties(this.columnArray, "style:table-column-properties");
    xmlWriter.closeStyle();
    
    function _properties(array, tagName) {
        if (array.length > 0) {
            xmlWriter.startOpenTag(tagName);
            for(let attr of array) {
                xmlWriter.addAttribute(attr.name, attr.value);
            }
            xmlWriter.closeEmptyTag();
        }
    }
};

OpenDocument.Style.build = function (element, styleName) {
    var styleFamily, parentStyleName, propertiesType;
    switch(element.tagName.toLowerCase()) {
        case "cell":
            styleFamily = "table-cell";
            propertiesType = "cell";
            break;
        case "row":
            styleFamily = "table-row";
            propertiesType = "row";
            break;
        case "column":
            styleFamily = "table-column";
            propertiesType = "column";
            break;
    }
    if (!styleName) {
        styleName = element.getAttribute("name");
    }
    parentStyleName = element.getAttribute("parent");
    var style = new OpenDocument.Style(styleName, styleFamily, parentStyleName);
    var cssText = $(element).text();
    var tokens = cssText.split(';');
    for(let token of tokens) {
        let paire = token.split(':');
        if (paire.length === 2) {
            let name = $.trim(paire[0]);
            let value = $.trim(paire[1]);
            if (OpenDocument.STYLE_ATTRIBUTES.hasOwnProperty(name)) {
                let attrDef = OpenDocument.STYLE_ATTRIBUTES[name];
                if (!attrDef.type) {
                    attrDef = Object.assign({}, attrDef, {type: propertiesType});
                }
                style.add(attrDef, value);
            }
        }
    }
    return style;
};